use crate::{gui::{main_tab::MainTab, MutexTIMManager, VRAM_HEIGHT, VRAM_WIDTH}, MainWindow};
use super::FileTab;
use rfd::FileDialog;
use slint::SharedString;
use tim_tool::logic::tim::types::Encoding;
use tool_helper::Error;

pub(super) fn on_browse_file(tim_manager: MutexTIMManager) -> impl FnMut(&mut FileTab, &MainWindow) -> Result<(), Error> + 'static {
    return move |file_tab, main_window| -> Result<(), Error> {
        let file = FileDialog::new()
            .add_filter("PNG image (.png)", &["png"])
            .set_title("PNG image file")
            .pick_file();

        if let Some(file) = file {
            if let Some(file_path) = file.to_str() {
                main_window.set_file_tab_browse_path(SharedString::from(file_path));
            }
            
            let file_name = if let Some(name) = file.file_name() {Some(name.to_string_lossy().to_string())} else {None};
            
            let (image, palette) = tim_manager.lock().expect("VRAM already locked").load_unadded_tim(&file)?;
        
            let img_size = image.size();
            if img_size.width > VRAM_WIDTH as u32 || img_size.height > VRAM_HEIGHT as u32 {
                return Err(Error::from_text(format!("Image size ({}; {}) is to big for VRAM ({}, {})", img_size.width, img_size.height, VRAM_WIDTH, VRAM_HEIGHT)));
            }
            return file_tab.update_new_loaded_file(file_name, image, palette);
        }

        Ok(())
    };    
}

pub(super) fn on_update_palette_size(tim_manager: MutexTIMManager) -> impl FnMut(&mut FileTab, &MainWindow, u32, u32) -> Result<(), Error> + 'static {
    return move |file_tab, _main_window, width, height| -> Result<(), Error> {
        file_tab.update_palette(tim_manager.lock().expect("VRAM already locked").change_unadded_tim_palette_size(width, height)?);
        Ok(())
    };
}

pub(super) fn on_add_image(tim_manager: MutexTIMManager) -> impl FnMut(&mut FileTab, &mut MainTab, &MainWindow) -> Result<(), Error> + 'static {
    return move |file_tab, main_tab, main_window| {
        let     file_name              = file_tab.get_file_name();
        let     encoding               = file_tab.get_encoding()?;
        let mut tim_mgr                = tim_manager.lock().expect("VRAM already locked");
        let     (image, palette_image) = tim_mgr.get_converted_unadded_tim_image(encoding)?;
        let     (full_image, _)        = tim_mgr.get_converted_unadded_tim_image(Encoding::FullColor)?;

        let images_created = main_tab.add_new_vram_file(&file_name, full_image, image, encoding, palette_image);
        if let Err(error) = tim_mgr.add_unadded_tim(images_created) {
            main_tab.pop_vram_files(images_created);
            return Err(error);
        }

        file_tab.clear_load();
        main_window.invoke_change_to_main();
        Ok(())
    };
}

pub(super) fn on_load_project_clicked() -> impl FnMut(&mut FileTab, &MainWindow) -> Result<(), Error> + 'static {
    move |_file_tab, _main_window| {
        Err(Error::not_implemented("on_load_project_clicked"))
    }
}

pub(super) fn on_save_project_clicked() -> impl FnMut(&mut FileTab, &MainWindow) -> Result<(), Error> + 'static {
    move |_file_tab, _main_window| {
        Err(Error::not_implemented("on_save_project_clicked"))
    }
}

pub(super) fn on_browse_open_project_clicked() -> impl FnMut(&mut FileTab, &MainWindow) -> Result<(), Error> + 'static {
    move |_file_tab, _main_window| {
        Err(Error::not_implemented("on_browse_open_project_clicked"))
    }
}

pub(super) fn on_browse_save_project_clicked() -> impl FnMut(&mut FileTab, &MainWindow) -> Result<(), Error> + 'static {
    move |_file_tab, _main_window| {
        Err(Error::not_implemented("on_browse_save_project_clicked"))
    }
}